'' =================================================================================================
''
''   File....... jm_Demo_Board_Cylon.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.term
''   Started.... 
''   Updated.... 21 DEC 2010
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1 = 31
  TX1 = 30
  SDA = 29
  SCL = 28


con 

  #1, HOME, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR, #16, CLS      ' PST formmatting control


obj

  term : "fullduplexserial"


var

  long  WavCog 

  long  stack[64]


pub main | t, idx, delay

  term.start(RX1, TX1, %0000, 115_200)
  pause(1)

  term.tx(CLS)
  term.str(string("By your command:", CR, "-- Merry Christmas!"))

  outa[23..16] := %00000000                                     ' leds off
  dira[23..16] := %11111111                                     ' demo board leds 

  delay := clkfreq / 1000 * (1506 / 14)                         ' file is 1.506s long

  repeat
    WavCog := cognew(playsound(10, @Cylon, 1), @stack) + 1
    t := cnt   
    repeat idx from 16 to 22                                    ' to
      outa[idx] := 1
      waitcnt(t += delay)
      outa[idx] := 0
    repeat idx from 23 to 17                                    ' fro
      outa[idx] := 1
      waitcnt(t += delay)
      outa[idx] := 0

    repeat while WavCog                                         ' let sound finish
    pause(1)                                                    ' let cog unload

 
pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)

                                                                
pri playsound(pin, pntr, plays) | rate, samples, addr, t        ' call with cognew

'' Plays 8-bit, mono audio file from DAT section
'' -- pin is pin for audio (uses CTRA)
'' -- pntr is start of audio (embedded WAV)
'' -- plays is number of plays before cog unloads itself

  if (plays =< 0)                                               ' if 0, play "forever"
    plays := posx

  rate    := clkfreq / long[pntr + 24]                          ' get rate from header
  samples := long[pntr + 40]                                    ' get samples from header

  frqa := $80 << 24                                             ' set to "0" 
  ctra := (%00110 << 26) + pin                                  ' duty cycle mode on pin
  dira[pin] := 1                                                ' make pin an output
  
  repeat plays
    addr := pntr + 44                                           ' skip over wav header
    t := cnt  
    repeat samples
      frqa := byte[addr++] << 24                                ' update dac
      waitcnt(t += rate)
      
  WavCog := 0                                                   ' alert forground
  cogstop(cogid)                                                ' unload cog


dat

                byte                                            ' force byte alignment
                
Cylon           file    "eye_11.wav"

                      
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}